/*
 * Version: 1.0
 *
 * The contents of this file are subject to the OpenVPMS License Version
 * 1.0 (the 'License'); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.openvpms.org/license/
 *
 * Software distributed under the License is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Copyright 2019 (C) OpenVPMS Ltd. All Rights Reserved.
 */

package org.openvpms.mapping.service;

import org.openvpms.component.model.object.IMObject;
import org.openvpms.mapping.model.Mappings;
import org.openvpms.mapping.model.Targets;

/**
 * Mapping service.
 *
 * @author Tim Anderson
 * @see Mappings
 * @see Targets
 */
public interface MappingService {

    /**
     * Create a configuration to record mappings between the specified archetype and a target.
     *
     * @param type the type being mapped
     * @return a new configuration to record mappings
     */
    <T extends IMObject> IMObject createMappingConfiguration(Class<T> type);

    /**
     * Creates mappings.
     *
     * @param config      the mapping configuration
     * @param type        the type being mapped
     * @param archetype   the archetype being mapped
     * @param displayName the mapping display name
     * @param targets     the targets
     * @return the mappings
     */
    <T extends IMObject> Mappings<T> createMappings(IMObject config, Class<T> type, String archetype,
                                                    String displayName, Targets targets);

    /**
     * Creates mappings.
     *
     * @param config            the mapping configuration
     * @param type              the type being mapped
     * @param archetype         the archetype being mapped
     * @param displayName       the mapping display name
     * @param sourceDisplayName the source object type display name
     * @param targets           the targets
     * @return the mappings
     */
    <T extends IMObject> Mappings<T> createMappings(IMObject config, Class<T> type, String archetype,
                                                    String displayName, String sourceDisplayName, Targets targets);

}
