/*
 * Version: 1.0
 *
 * The contents of this file are subject to the OpenVPMS License Version
 * 1.0 (the 'License'); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.openvpms.org/license/
 *
 * Software distributed under the License is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Copyright 2019 (C) OpenVPMS Ltd. All Rights Reserved.
 */

package org.openvpms.web.workspace.admin.mapping;

import org.apache.commons.lang.ObjectUtils;
import org.openvpms.component.model.object.IMObject;
import org.openvpms.mapping.model.Mapping;
import org.openvpms.mapping.model.Mappings;
import org.openvpms.mapping.model.Source;
import org.openvpms.mapping.model.Target;
import org.openvpms.web.component.edit.Editors;
import org.openvpms.web.component.im.layout.LayoutContext;
import org.openvpms.web.component.im.util.IMObjectHelper;
import org.openvpms.web.component.property.Modifiable;

/**
 * .
 *
 * @author Tim Anderson
 */
class MappingEditor<T extends IMObject> extends Editors {

    private final MappingState mapping;

    private final Mappings<T> mappings;

    private final SourceEditor source;

    private final TargetEditor target;

    MappingEditor(MappingState mapping, Mappings<T> mappings, LayoutContext context) {
        this.mapping = mapping;
        this.mappings = mappings;
        source = new SourceEditor(mapping.getSource(), mappings, context);
        target = new TargetEditor(mapping.getTarget(), mappings, context);
        add(source);
        add(target);
    }

    public SourceEditor getSource() {
        return source;
    }

    public TargetEditor getTarget() {
        return target;
    }

    /**
     * Invoked when a {@link Modifiable} changes. Resets the cached valid status and forwards the event to any
     * registered listener.
     *
     * @param modified the changed instance
     */
    @Override
    protected void onModified(Modifiable modified) {
        Source newSource = source.getObject();
        Target newTarget = target.getObject();
        if (newSource != null && newTarget != null) {
            if (!ObjectUtils.equals(newSource.getId(), mapping.getSource())
                || !ObjectUtils.equals(newTarget, mapping.getTarget())) {
                Class<T> type = mappings.getType();
                T object = IMObjectHelper.getObject(newSource.getId(), type);
                if (mapping.getMapping() == null) {
                    Mapping newMapping = mappings.add(object, newTarget);
                    mapping.setMapping(newMapping);
                } else {
                    mappings.replace(mapping.getMapping(), object, newTarget);
                }
            }
        } else if (mapping.getMapping() != null) {
            Mapping existing = mapping.getMapping();
            mappings.remove(existing);
            mapping.setMapping(null);
        }
        super.onModified(modified);

    }
}
