/*
 * Version: 1.0
 *
 * The contents of this file are subject to the OpenVPMS License Version
 * 1.0 (the 'License'); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.openvpms.org/license/
 *
 * Software distributed under the License is distributed on an 'AS IS' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * Copyright 2019 (C) OpenVPMS Ltd. All Rights Reserved.
 */

package org.openvpms.web.workspace.admin.mapping;

import org.openvpms.component.model.object.Reference;
import org.openvpms.mapping.model.Mapping;
import org.openvpms.mapping.model.Mappings;
import org.openvpms.mapping.model.Source;
import org.openvpms.mapping.model.Target;

import java.util.ArrayList;
import java.util.List;

/**
 * .
 *
 * @author Tim Anderson
 */
public class MappingResultSet extends AbstractMappingResultSet<Mapping> {

    private List<Target> unmapped;

    private final Mappings<?> mappings;

    /**
     * Constructs a {@link MappingResultSet}.
     *
     * @param pageSize the maximum no. of results per page
     * @param mappings the mappings
     */
    MappingResultSet(String value, int pageSize, Mappings<?> mappings) {
        super(value, pageSize);
        this.mappings = mappings;
    }

    @Override
    protected List<Mapping> getMatches(String value, int firstResult, int maxResults) {
        List<Mapping> result = new ArrayList<>();
        getUnmapped(value);
        int size = unmapped.size();
        if (firstResult < size) {
            for (int i = firstResult; i < size; ++i) {
                result.add(new MappingState(null, unmapped.get(i), null));
            }
            if (firstResult + maxResults > size) {
                int remainder = maxResults - size;
                getSources(value, 0, remainder, result);
            }
        } else {
            firstResult -= unmapped.size();
            getSources(value, firstResult, maxResults, result);
        }
        return result;
    }

    /**
     * Counts the no. of results matching the query criteria.
     *
     * @param value the value to filter on. May be {@code null}
     * @return the total number of results
     */
    @Override
    protected int countResults(String value) {
        getUnmapped(value);
        int count = (int) mappings.getSourceCount(value, false);
        count += unmapped.size();
        return count;
    }

    private void getSources(String value, int firstResult, int maxResults, List<Mapping> result) {
        for (Source source : mappings.getSources(value, false, firstResult, maxResults)) {
            Reference id = source.getId();
            Mapping mapping = mappings.getMapping(id);
            Target target = (mapping != null) ? mapping.getTarget() : null;
            result.add(new MappingState(id, target, mapping));
        }
    }

    private void getUnmapped(String value) {
        if (unmapped == null) {
            unmapped = mappings.getTargets(value, true, 0, -1);
        }
    }

}
